package weka.classifiers.functions;

import Jama.Matrix;

public class LinearRegression_AdditiveLogLoss extends CustomLinearRegression {

	private double	k = 1;//parameter of our loss function.
	
	private static final long serialVersionUID = -6548614236040986416L;
	
	/**
	   * Returns a string describing this classifier
	   * @return a description of the classifier suitable for
	   * displaying in the explorer/experimenter gui
	   */
	  public String globalInfo() {
	    return "This classifier uses square loss to fit a hyperplane to the data";
	  }
	  
	protected double getErrorFunction() {
		double f = 0;

		/* find the value of error function for current w */
		Matrix predictedY = x.times(w);
		Matrix error = y.minus(predictedY);
		error.timesEquals(k);
		double	maxErr = Math.log(Double.MAX_VALUE);
		for (int i = 0; i < error.getRowDimension(); i++)
		{
			double	err = error.get(i, 0);
			
			if(Math.abs(err) < maxErr)
			{
				f += Math.log( 1 + Math.exp(err)) +  Math.log(1 + Math.exp(-err) );
			}
			else
			{
//				System.err.print(" BIG ");
				f += Math.abs(err);
			}
			
		}
		
		return f;
		
	}

	protected Matrix getGradient() {
		/* find the value of error function for current w */
		Matrix predictedY = x.times(w);
		Matrix error = y.minus(predictedY);
		error.timesEquals(k);
		double	maxErr = Math.log(Double.MAX_VALUE);
		Matrix gradient = new Matrix(1, ndim);
		for (int i = 0; i < error.getRowDimension(); i++)
		{
			double	err = error.get(i, 0);
				
			if(Math.abs(err) >= maxErr)
			{
//				System.err.print(" BIG ");
				gradient.minusEquals(x.getMatrix(i,i,0,x.getColumnDimension()-1).times( k ));
			}
			else
			{
				double XX =	(1.0 / (1 + (Math.exp(-k * err)))) - (1.0 / (1 + (Math.exp(k * err))));
				gradient.minusEquals(x.getMatrix(i,i,0,x.getColumnDimension()-1).times( k * XX ));
			}
			
		}
		return gradient.transpose();	
	}

	
	
	public static void main(String [] argv){
		runClassifier(new LinearRegression_AdditiveLogLoss(), argv);
	} 	  
}


//protected double getErrorFunction() {
//  	Matrix predictedY = x.times(w);
//	Matrix error = y.minus(predictedY);
//	double	maxErr = Math.log(Double.MAX_VALUE);
//	double f=0;
//	for (int i = 0; i < error.getRowDimension(); i++)
//	{
//		double	err = Math.abs(error.get(i, 0))/scaleFactor;
//		//if(err < maxErr)
//		//{
//			f += Math.log( Math.exp(err) +  Math.exp(-err) );
//		//}
//		//else
//		//{
//		//	f += (err);
//		//}		
//	}
//	return f;
//}
//
//protected Matrix getGradient() {
//  	Matrix predictedY = x.times(w);
//	Matrix error = y.minus(predictedY);
//	Matrix gradient = new Matrix(1, ndim);
//	for (int i = 0; i < error.getRowDimension(); i++)
//	{
//		double	err = Math.abs(error.get(i, 0))/scaleFactor;
//		gradient.minusEquals(x.getMatrix(i,i,0,x.getColumnDimension()-1).times( Math.tanh( err ) ));
//	}
//	
//	return gradient.transpose();
//}
